--[[
AdiFocusBar - Smart focus bar for hunters.
Copyright 2010-2013 Adirelle (adirelle@gmail.com)
All rights reserved.
--]]

if select(2, UnitClass("player")) ~= "HUNTER" then return end

local addonName, ns = ...

-- Setup the debugging facility
if AdiDebug then
	ns.Debug = AdiDebug:GetSink(addonName)
else
	ns.Debug = function() end
end
local Debug = ns.Debug

-- Create the initial frame
local addon = CreateFrame("Frame", addonName, UIParent)
ns.addon = addon

addon:SetScript('OnEvent', function(self, event, ...)
	return self[event](self, event, ...) 
end)

addon:RegisterEvent('ADDON_LOADED')
function addon:ADDON_LOADED(event, name)
	if name ~= addonName then return end
	self:UnregisterEvent("ADDON_LOADED")

	if not AdiFocusBarDB then
		AdiFocusBarDB = { layout = {} }
	end

	self:SetupGUI()
	self:Hide()

	return self:TestVisibility(event)
end

local majorTargets = { worldboss = true, rareelite = true }

function addon:TestVisibility(event)
	local specGroup = GetSpecialization()
	local spec = specGroup and GetSpecializationInfo(specGroup)
	local hide = nil
	if not spec then
		hide = 'No spec'
	elseif not UnitAffectingCombat("player") then
		hide = 'Not in combat'
	elseif not UnitExists("target") or UnitIsDead("target") or not UnitCanAttack("player", "target") then
		hide = 'No valid target'
	elseif not UnitExists("boss1") and not majorTargets[UnitClassification("target")] then
		hide = 'Minor target'
	else
		if not self:IsShown() then
			Debug('Valid target: show')
			return self:Show()
		end
	end
	if self:IsShown() then
		Debug(hide..': hide')
		return self:Hide()
	end
end
addon.PLAYER_TARGET_CHANGED = addon.TestVisibility
addon.PLAYER_REGEN_DISABLED = addon.TestVisibility
addon.PLAYER_REGEN_ENABLED = addon.TestVisibility

function addon:OnShow()
	self:RegisterUnitEvent('UNIT_HEALTH_FREQUENT', 'target')
	self:RegisterUnitEvent('UNIT_POWER_FREQUENT', 'player')
	self:RegisterUnitEvent('UNIT_MAXPOWER', 'player')
	self:RegisterUnitEvent('UNIT_STATS', 'player')
	self:RegisterEvent('UNIT_AURA')
	self:RegisterEvent('SPELLS_CHANGED')
	self:RegisterEvent('PLAYER_TALENT_UPDATE')
	self:RegisterUnitEvent('UNIT_SPELLCAST_START', 'player')
	self:RegisterUnitEvent('UNIT_SPELLCAST_STOP', 'player')
	self:RegisterUnitEvent('UNIT_SPELLCAST_SUCCEEDED', 'player')
	self:RegisterEvent('SPELL_UPDATE_COOLDOWN')
	self:SPELLS_CHANGED()
	return self:Update("OnShow")
end

function addon:OnHide()
	self:UnregisterAllEvents()
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end
